@@ -75,12 +75,16 @@ def affine_transform(txy=(0,0), r=0, sx=1, sy=1, kx=0, ky=0):
     
     Usage:
     
-    >>> A = vipy.geometry.affine_transform(r=np.pi/4)
-    >>> vipy.image.Image(array=vipy.geometry.imtransform(im.array(), A), colorspace='float')
+    ```python
+    A = vipy.geometry.affine_transform(r=np.pi/4)
+    vipy.image.Image(array=vipy.geometry.imtransform(im.array(), A), colorspace='float')
+    ```
     
     Equivalently:
 
-    >>> im = vipy.image.RandomImage().affine_transform(A)    
+    ```python
+    im = vipy.image.RandomImage().affine_transform(A)    
+    ```
     
     """
     assert istuple(txy) and len(txy) == 2 and isnumber(r) and isnumber(sx) and isnumber(sy) and isnumber(kx) and isnumber(ky), "Invalid input"
@@ -564,7 +568,15 @@ class BoundingBox(object):
         return self.area() + bb.area() - self.area_of_intersection(bb)
         
     def cover(self, bb):
-        """Fraction of this bounding box intersected by other bbox (bb)"""
+        """Fraction of this bounding box intersected by other bbox (bb).
+
+        .. note:: 
+        
+            - Cover is often more useful than `vipy.geometry.BoundingBox.iou` as a measure of overlap due to bounding box distortion from partially occluded object proposals.  
+            - For example, an object proposal of a person may generate a smaller box (e.g. just the torso) when the lower body is occluded whereas a track will have the full body box.  
+            - `vipy.geometry.BoundingBox.maxcover` is a better measure of assignment in this case.  
+
+        """
         a = float(self.area())
         return (self.area_of_intersection(bb) / a) if a>0 else 0
 
@@ -919,12 +931,18 @@ class BoundingBox(object):
         return np.abs(self.bbwidth()-other.bbwidth())  + np.abs(self.bbheight()-other.bbheight())
 
     def affine(self, A):
-        """Apply an 2x3 affine transformation to the box centroid.  This operation preserves an axis aligned bounding box for an arbitrary affine transform."""
+        """Apply an 2x3 affine transformation to the box centroid.  
+
+        .. note::  This transformation is performed on the centroid and not the box corners, so the box will still be rectilinear after the transform
+        """
         assert isnumpy(A) and A.shape == (2,3), "A must be a 2x3 affine transformation matrix"
         return self.centroid(np.dot(A, homogenize(np.array(self.centroid()))))
 
     def projective(self, A):
-        """Apply an 3x3 affine transformation to the box centroid.  This operation preserves an axis aligned bounding box for an arbitrary affine transform."""
+        """Apply an 3x3 projective transformation to the box centroid.  
+        
+        .. note:: This transformation is performed on the centroid and not the box corners, so the box will still be rectilinear after the transform
+        """
         assert isnumpy(A) and A.shape == (3,3), "A must be a 3x3 affine transformation matrix"
         return self.centroid(dehomogenize(np.dot(A, homogenize(np.array(self.centroid())))))
     
