@@ -11,6 +11,7 @@ try:
 except ImportError:
     import json
 
+
 DETECTION_GUID = int(uuid.uuid4().hex[0:8], 16)  
 
 class Detection(BoundingBox):
@@ -19,11 +20,13 @@ class Detection(BoundingBox):
     This class represent a single object detection in the form a bounding box with a label and confidence.
     The constructor of this class follows a subset of the constructor patterns of vipy.geometry.BoundingBox
 
-    >>> d = vipy.object.Detection(category='Person', xmin=0, ymin=0, width=50, height=100)
-    >>> d = vipy.object.Detection(label='Person', xmin=0, ymin=0, width=50, height=100)  # "label" is an alias for "category"
-    >>> d = vipy.object.Detection(label='John Doe', shortlabel='Person', xmin=0, ymin=0, width=50, height=100)  # shortlabel is displayed
-    >>> d = vipy.object.Detection(label='Person', xywh=[0,0,50,100])
-    >>> d = vupy.object.Detection(..., id=True)  # generate a unique UUID for this detection retrievable with d.id()
+    ```python
+    d = vipy.object.Detection(category='Person', xmin=0, ymin=0, width=50, height=100)
+    d = vipy.object.Detection(label='Person', xmin=0, ymin=0, width=50, height=100)  # "label" is an alias for "category"
+    d = vipy.object.Detection(label='John Doe', shortlabel='Person', xmin=0, ymin=0, width=50, height=100)  # shortlabel is displayed
+    d = vipy.object.Detection(label='Person', xywh=[0,0,50,100])
+    d = vupy.object.Detection(..., id=True)  # generate a unique UUID for this detection retrievable with d.id()
+    ```
 
     """
 
@@ -105,9 +108,36 @@ class Detection(BoundingBox):
         self._shortlabel = None
         return self
         
-    def category(self, category=None, shortlabel=True):
+    def categoryif(self, ifcategory, tocategory=None):
+        """If the current category is equal to ifcategory, then change it to newcategory.
+
+        Args:
+            
+            ifcategory [dict, str]: May be a dictionary {ifcategory:tocategory}, or just an ifcategory
+            tocategory [str]:  the target category 
+
+        Returns:
+        
+            this object with the category changed.
+
+        .. note:: This is useful for converting synonyms such as self.categoryif('motorbike', 'motorcycle')
+        """
+        assert (isinstance(ifcategory, dict) and tocategory is None) or tocategory is not None
+
+        if isinstance(ifcategory, dict):
+            for (k,v) in ifcategory.items():
+                self.categoryif(k, v)
+        elif self.category() == ifcategory:
+            self.category(tocategory, shortlabel=False, capitalize=False)
+        return self
+
+    def category(self, category=None, shortlabel=True, capitalize=False):
         """Update the category and shortlabel (optional) of the detection"""
-        if category is None:
+        if capitalize:
+            self._label = self._label.capitalize()
+            self._shortlabel = self._shortlabel.capitalize() if shortlabel else self._shortlabel
+            return self
+        elif category is None:
             return self._label
         else:
             self._label = str(category)  # coerce to string
@@ -172,19 +202,25 @@ class Track(object):
 
     Valid constructors are:
 
-    >>> t = vipy.object.Track(keyframes=[0,100], boxes=[vipy.geometry.BoundingBox(0,0,10,10), vipy.geometry.BoundingBox(0,0,20,20)], label='Person')
-    >>> t = vipy.object.Track(keyframes=[0,100], boxes=[vipy.geometry.BoundingBox(0,0,10,10), vipy.geometry.BoundingBox(0,0,20,20)], label='Person', interpolation='linear')
-    >>> t = vipy.object.Track(keyframes=[10,100], boxes=[vipy.geometry.BoundingBox(0,0,10,10), vipy.geometry.BoundingBox(0,0,20,20)], label='Person', boundary='strict')
+    ```python
+    t = vipy.object.Track(keyframes=[0,100], boxes=[vipy.geometry.BoundingBox(0,0,10,10), vipy.geometry.BoundingBox(0,0,20,20)], label='Person')
+    t = vipy.object.Track(keyframes=[0,100], boxes=[vipy.geometry.BoundingBox(0,0,10,10), vipy.geometry.BoundingBox(0,0,20,20)], label='Person', interpolation='linear')
+    t = vipy.object.Track(keyframes=[10,100], boxes=[vipy.geometry.BoundingBox(0,0,10,10), vipy.geometry.BoundingBox(0,0,20,20)], label='Person', boundary='strict')
+    ```
 
     Tracks can be constructed incrementally:
 
-    >>> t = vipy.object.Track('Person')
-    >>> t.add(0, vipy.geometry.BoundingBox(0,0,10,10))
-    >>> t.add(100, vipy.geometry.BoundingBox(0,0,20,20))
+    ```python
+    t = vipy.object.Track('Person')
+    t.add(0, vipy.geometry.BoundingBox(0,0,10,10))
+    t.add(100, vipy.geometry.BoundingBox(0,0,20,20))
+    ```
 
     Tracks can be resampled at a new framerate, as long as the framerate is known when the keyframes are extracted
 
-    >>> t.framerate(newfps)
+    ```python
+    t.framerate(newfps)
+    ```
 
     """
 
@@ -204,7 +240,7 @@ class Track(object):
         self._id = uuid.uuid4().hex if trackid is None else trackid
         self._label = category if category is not None else label
         self._shortlabel = self._label if shortlabel is None else shortlabel
-        self._framerate = framerate
+        self._framerate = float(framerate) if framerate is not None else framerate
         self._interpolation = interpolation
         self._boundary = boundary
         self.attributes = attributes.copy() if attributes is not None else {}  # shallow copy
@@ -382,7 +418,7 @@ class Track(object):
 
         .. warning::  This should really only be set by the user in the constructor and is included here as an admin override for some legacy JSON that did not contain framerates.  Use with caution!
         """
-        self._framerate = fps
+        self._framerate = float(fps)
         return self
 
     def framerate(self, fps=None, speed=None):
@@ -404,7 +440,7 @@ class Track(object):
         assert not (fps is not None and speed is not None), "Invalid input"
         assert speed is None or speed > 0, "Invalid speed, must specify speed multiplier s=1, s=2 for 2x faster, s=0.5 for half slower"
         
-        fps = fps if fps is not None else (1.0/speed)*self._framerate
+        fps = float(fps) if fps is not None else (1.0/speed)*self._framerate
         self._keyframes = [int(np.round(f*(fps/float(self._framerate)))) for f in self._keyframes]
         self._framerate = fps
         return self
@@ -424,6 +460,15 @@ class Track(object):
         """
         return self._keyframes[-1] if len(self._keyframes)>0 else None  # assumes sorted order
 
+    def duration(self):
+        """The length of the track in seconds.
+
+        Returns:
+            The duration in seconds of this track object
+        """
+        assert self.framerate() is not None, "Framerate must be set in constructor"
+        return len(self) / float(self.framerate())
+    
     def linear_interpolation(self, f, id=True):
         """Linear bounding box interpolation at frame=k given observed boxes (x,y,w,h) at keyframes.  
 
@@ -458,6 +503,7 @@ class Track(object):
                       shortlabel=self.shortlabel(),
                       id=id)
         d.attributes['trackid'] = self.id()  # for correspondence of detections to tracks
+        d.attributes['__trackid'] = d.attributes['trackid'] # trackid to be deprecated
         return d if self._boundary == 'extend' or self.during(f) else None
 
     def category(self, label=None, shortlabel=True):
@@ -471,6 +517,29 @@ class Track(object):
         else:
             return self._label
     
+    def categoryif(self, ifcategory, tocategory=None):
+        """If the current category is equal to ifcategory, then change it to newcategory.
+
+        Args:
+            
+            ifcategory [dict, str]: May be a dictionary {ifcategory:tocategory}, or just an ifcategory
+            tocategory [str]:  the target category 
+
+        Returns:
+        
+            this object with the category changed.
+
+        .. note:: This is useful for converting synonyms such as self.categoryif('motorbike', 'motorcycle')
+        """
+        assert (isinstance(ifcategory, dict) and tocategory is None) or tocategory is not None
+
+        if isinstance(ifcategory, dict):
+            for (k,v) in ifcategory.items():
+                self.categoryif(k, v)
+        elif self.category() == ifcategory:
+            self.category(tocategory, shortlabel=False)
+        return self
+
     def label(self, label):
         """Alias for category"""
         return self.category(label, shortlabel=True)
